from approvaltests import Options
from approvaltests.utilities.command_line_approvals import verify_command_line


def test_emoji_output() -> None:
    """
    Hello 🌟 World 🎉 Test 🚀
    """
    verify_command_line(
        "python -c \"print('Hello 🌟 World 🎉 Test 🚀')\"", options=Options().inline()
    )



import itertools
from typing import Any, Callable, Dict, Generator, Sequence


def first(sequence: Sequence[Any], predicate: Callable[[Any], bool]) -> Any:
    matching = filter(predicate, sequence)
    return next(matching, None)


def product_dict(**kwargs: Sequence[Any]) -> Generator[Dict[str, Any], None, None]:
    """
    Similar to `itertools.product`, but the resulting combinations retain the names.
    """
    keys = kwargs.keys()
    vals = kwargs.values()
    for instance in itertools.product(*vals):
        yield dict(zip(keys, instance))



import textwrap
from typing import Optional


def remove_indentation_from(text: Optional[str]) -> str:
    if not text:
        return ""
    cleaned = textwrap.dedent(text + "|")
    if cleaned.startswith("\n"):
        cleaned = cleaned[1:]
    return cleaned[:-1]



import os
from types import TracebackType
from typing import Optional, Type

from typing_extensions import ContextManager


def use_utc_timezone() -> ContextManager[None]:
    class TimeZoneSwap:
        def __init__(self) -> None:
            self.timezone: Optional[str] = ""

        def __enter__(self) -> None:
            self.timezone = os.environ.get("TZ")
            os.environ["TZ"] = "UTC"

        def __exit__(
            self,
            exc_type: Optional[Type[BaseException]],
            exc_val: Optional[BaseException],
            exc_tb: Optional[TracebackType],
        ) -> bool:
            if self.timezone is None:
                os.environ.pop("TZ")
            else:
                os.environ["TZ"] = self.timezone

    return TimeZoneSwap()



from typing import Callable

from approvaltests import Options, verify_all
from approvaltests.inline.parse import Parse


def test_single_strings() -> None:
    """
    Sam -> SAM
    Llewellyn -> LLEWELLYN
    """
    parse = Parse.doc_string()
    verify_all(
        "",
        parse.get_inputs(),
        lambda s: f"{s} -> {s.upper()}",
        options=Options().inline(),
    )
    parse.verify_all(lambda s: s.upper())


def test_with_types_transformers_and_both() -> None:
    """
    1 -> 0b1
    9 -> 0b1001
    """
    parse = Parse.doc_string()
    verify_all(
        "",
        parse.get_inputs(),
        lambda s: f"{s} -> {bin(int(s))}",
        options=Options().inline(),
    )
    parse.transform(lambda a: int(a)).verify_all(lambda i: bin(i))


# parse.transform(int).transform(str).transform(int).verify_all(lambda i: bin(i), options= Options().with_reporter(ReporterThatAutomaticallyApproves())


def test_with_2_types_transformers_and_both() -> None:
    """
    1, 5.0 -> 5.0
    4, 0.5 -> 2.0
    """
    parse = Parse.doc_string(auto_approve=True)
    parse.transform2(int, float).verify_all(lambda i, f: i * f)
    parse.transform2(str, str).transform2(int, float).verify_all(lambda i, f: i * f)


def test_example_step_1() -> None:
    # begin-snippet: parse_input_step_2
    """
    Kody -> 0
    """

    # end-snippet
    # begin-snippet: parse_input_step_1
    def count_vowels(s: str) -> int:
        return 0

    def test_count_vowels():
        """
        Kody
        """
        parse = Parse.doc_string(auto_approve=True)
        parse.verify_all(count_vowels)

    # end-snippet
    test_count_vowels()


def test_example_step_2() -> None:
    """
    Kody -> 1
    Teresa -> 3
    Green -> 2
    """

    # begin-snippet: parse_input_step_3
    def count_vowels(s: str) -> int:
        return sum(1 for c in s if c in "aeo")

    def test_count_vowels():
        """
        Kody -> 1
        Teresa -> 3
        Green -> 2
        """
        parse = Parse.doc_string(auto_approve=True)
        parse.verify_all(count_vowels)

    # end-snippet
    test_count_vowels()


# begin-snippet: parse_input_transformation
def test_with_transformation() -> None:
    """
    1 -> 0b1
    9 -> 0b1001
    """
    parse = Parse.doc_string(auto_approve=True)
    parse.transform(int).verify_all(bin)


# end-snippet


# begin-snippet: parse_input_two_parameters
def test_with_two_parameters() -> None:
    """
    a, 3 -> aaa
    !, 7 -> !!!!!!!
    """
    parse = Parse.doc_string(auto_approve=True)
    parse.transform2(str, int).verify_all(lambda s, i: s * i)


# end-snippet


# long-term intention: get the test to fail, randomSauce is incorrect
def test_with_3_parameters() -> None:
    """
    a, 3, 1 -> aaaa
    """

    counter = 0

    def to(tin: type, tout: type) -> Callable:
        def wrapped(input):
            nonlocal counter
            counter += 1
            assert type(input) is tin
            return tout(input)

        return wrapped

    parse = Parse.doc_string(auto_approve=True)
    parse.transform3(to(str, str), to(str, str), to(str, str)).transform3(
        to(str, str), to(str, int), to(str, int)
    ).verify_all(lambda a, b, c: to(str, str)(a) * (to(int, int)(b) + to(int, int)(c)))

    assert counter == 9


# assert on all the paramaters



from approvaltests import Options, verify_all


def test_verify_all_with_no_header() -> None:
    """
    0) 1
    1) 2
    2) 3
    """
    verify_all("", ["1", "2", "3"], options=Options().inline())



import threading
from typing import Any, Callable, ContextManager, Optional, Union

from approval_utilities.utilities.logger.logging_instance import LoggingInstance
from approval_utilities.utilities.string_wrapper import StringWrapper
from approval_utilities.utilities.wrapper import SingleWrapper, ThreadedWrapper, Wrapper


class SimpleLogger:
    _wrapper: Wrapper[LoggingInstance] = SingleWrapper(LoggingInstance())

    @staticmethod
    def register_logger(log_method: Callable[[str], None]) -> None:
        SimpleLogger._wrapper.get().logger = log_method

    @staticmethod
    def log_to_string(log_separate_threads: bool = True) -> StringWrapper:
        with threading.Lock():
            if log_separate_threads and not isinstance(
                SimpleLogger._wrapper, ThreadedWrapper
            ):
                SimpleLogger._wrapper = ThreadedWrapper(lambda: LoggingInstance())
        return SimpleLogger._wrapper.get().log_to_string()

    @staticmethod
    def use_markers(
        parameter_text: Optional[Union[str, Callable[[], str]]] = None,
    ) -> ContextManager[None]:
        return SimpleLogger._wrapper.get().use_markers(
            parameter_text, additional_stack=1
        )

    @staticmethod
    def variable(name: str, value: Any, show_types: bool = False) -> None:
        SimpleLogger._wrapper.get().variable(name, value, show_types=show_types)

    @staticmethod
    def hour_glass() -> None:
        SimpleLogger._wrapper.get().hour_glass()

    @staticmethod
    def event(event_name: str) -> None:
        SimpleLogger._wrapper.get().event(event_name)

    @staticmethod
    def show_timestamps(display: bool) -> None:
        SimpleLogger._wrapper.get().log_with_timestamps = display

    @staticmethod
    def query(query_text: str) -> None:
        SimpleLogger._wrapper.get().query(query_text)

    @staticmethod
    def message(message: str) -> None:
        SimpleLogger._wrapper.get().message(message)

    @staticmethod
    def warning(
        text: Union[str, BaseException] = "", exception: Optional[BaseException] = None
    ) -> None:
        SimpleLogger._wrapper.get().warning(text, exception)

    @staticmethod
    def show_queries(show: bool) -> None:
        SimpleLogger._wrapper.get().show_queries(show)

    @staticmethod
    def show_all(show: bool) -> None:
        SimpleLogger._wrapper.get().show_all(show)

    @staticmethod
    def show_messages(show: bool) -> None:
        SimpleLogger._wrapper.get().show_messages(show)

    @staticmethod
    def show_variables(show: bool) -> None:
        SimpleLogger._wrapper.get().show_variables(show)

    @staticmethod
    def show_hour_glass(show: bool) -> None:
        SimpleLogger._wrapper.get().show_hour_glass(show)

    @staticmethod
    def show_markers(show: bool) -> None:
        SimpleLogger._wrapper.get().show_markers(show)

    @staticmethod
    def show_events(show: bool) -> None:
        SimpleLogger._wrapper.get().show_events(show)



import inspect
from typing import Any, Dict, Sequence

from approvaltests.combination_approvals import (
    verify_all_combinations,
    verify_best_covering_pairs,
)
from approvaltests.pairwise_combinations import get_best_covering_pairs


def get_key(in1: Any, in2: Any) -> str:
    return f"({in1},{in2})"


def add_pair(in1: Any, in2: Any, all_pairs_with_index: Dict[str, int]) -> None:
    key = get_key(in1, in2)
    all_pairs_with_index.setdefault(key, len(all_pairs_with_index))


def get_all_pairs_count(inputs: Sequence[Sequence[Any]]) -> Dict[str, int]:
    cases = get_best_covering_pairs(inputs)
    print(f"reduce set of {len(cases)} cases")
    pairCount: Dict[str, int] = {}
    for params in cases:
        for i1 in range(len(inputs) - 1):
            for i2 in range(i1 + 1, len(inputs)):
                in1 = params[i1]
                in2 = params[i2]
                add_pair(in1, in2, pairCount)
    return pairCount


def assert_all_pairs_present_between_lists(
    input1: Sequence[Any], input2: Sequence[Any], all_pairs_with_index: Dict[str, int]
) -> None:
    for i1 in input1:
        for i2 in input2:
            key = get_key(i1, i2)
            if key not in all_pairs_with_index.keys():
                raise Exception(f"could not find pair {key}")


def assert_all_pairs_present(
    all_pairs_with_index: Dict[str, int], inputs: Sequence[Sequence[Any]]
) -> None:
    for i1 in range(len(inputs) - 1):
        for i2 in range(i1 + 1, len(inputs)):
            assert_all_pairs_present_between_lists(
                inputs[i1], inputs[i2], all_pairs_with_index
            )


def assert_pairwise_combinations(inputs: Sequence[Sequence[Any]]) -> None:
    all_pairs_with_index = get_all_pairs_count(inputs)
    assert_all_pairs_present(all_pairs_with_index, inputs)


def test_pair_properties() -> None:
    input1 = [112, 111, 113, 114, 115]
    input2 = [221, 222, 223, 224, 225]
    input3 = [331, 332, 333, 334, 335]
    input4 = [441, 442, 443, 444, 445]
    assert_pairwise_combinations([input1, input2, input3, input4])


def test_reduction() -> None:
    inputs1 = list(range(1, 10))
    inputs2 = list(range(11, 20))
    inputs3 = list(range(21, 30))
    inputs4 = list(range(31, 40))
    inputs5 = list(range(41, 50))
    inputs6 = list(range(51, 60))
    inputs7 = list(range(61, 70))
    inputs8 = list(range(71, 80))
    inputs9 = list(range(81, 90))
    assert_pairwise_combinations(
        [
            inputs1,
            inputs2,
            inputs3,
            inputs4,
            inputs5,
            inputs6,
            inputs7,
            inputs8,
            inputs9,
        ]
    )
    verify_best_covering_pairs(
        lambda *args: sum(args),
        [
            inputs1,
            inputs2,
            inputs3,
            inputs4,
            inputs5,
            inputs6,
            inputs7,
            inputs8,
            inputs9,
        ],
    )


def test_signature_equality() -> None:
    assert inspect.signature(verify_all_combinations) == inspect.signature(
        verify_best_covering_pairs
    )



import functools
import warnings
from typing import Callable


def deprecated(reason: str) -> Callable:
    def decorator(func: Callable) -> Callable:
        """This is a decorator which can be used to mark functions
        as deprecated. It will result in a warning being emitted
        when the function is used."""

        @functools.wraps(func)
        def new_func(*args, **kwargs):
            warnings.simplefilter("always", DeprecationWarning)  # turn off filter
            warnings.warn(
                f"Call to deprecated function {func.__name__}. {reason}",
                category=DeprecationWarning,
                stacklevel=2,
            )
            warnings.simplefilter("default", DeprecationWarning)  # reset filter
            return func(*args, **kwargs)

        return new_func

    return decorator



from abc import ABC, abstractmethod


class CommandGetter(ABC):
    @abstractmethod
    def get_command(self) -> str:
        raise NotImplementedError("Interface member not implemented")


class CommandExecutor(ABC):
    @abstractmethod
    def execute_command(self, command: str) -> str:
        raise NotImplementedError("Interface member not implemented")


class ExecutableCommand(CommandGetter, CommandExecutor):
    pass



from typing import TYPE_CHECKING, Any, Callable, Iterable

from typing_extensions import override

from approval_utilities.approvaltests.core.verifiable import Verifiable
from approval_utilities.approvaltests.core.verify_parameters import VerifyParameters

if TYPE_CHECKING:
    from approvaltests.core.options import Options


class MarkdownTable(Verifiable):
    def __init__(self) -> None:
        self.markdown: str = ""

    @override
    def get_verify_parameters(self, options: "Options") -> VerifyParameters:
        return VerifyParameters(options.for_file.with_extension(".md"))

    @staticmethod
    def with_headers(*column_names: str) -> "MarkdownTable":
        table = MarkdownTable()
        table.add_rows(*column_names)
        dividers = map(lambda _: "---", column_names)
        table.add_rows(*dividers)
        return table

    def add_rows(self, *column_names: str) -> "MarkdownTable":
        self.markdown += MarkdownTable.print_row(*column_names)
        return self

    @override
    def __str__(self) -> str:
        return self.markdown

    @staticmethod
    def print_row(*column_names: Any) -> str:
        row = "| " + " | ".join(map(str, column_names)) + " |"
        return row + "\n"

    def add_rows_for_inputs(
        self, inputs: Iterable[Any], *input_transformers: Callable[[Any], Any]
    ) -> "MarkdownTable":
        def transform_resolver_for_input(
            input_value: Any,
        ) -> Callable[[Callable[[Any], Any]], Any]:
            return lambda transform: transform(input_value)

        for row_input in inputs:
            row = [row_input]
            row += map(transform_resolver_for_input(row_input), input_transformers)
            self.add_rows(*row)
        return self



from unittest import TestCase

from typing_extensions import override

from approvaltests import approvals
from approvaltests.reporters.clipboard_reporter import CommandLineReporter
from approvaltests.reporters.diff_reporter import DiffReporter
from approvaltests.reporters.generic_diff_reporter import GenericDiffReporter
from approvaltests.reporters.multi_reporter import MultiReporter


class TestList(TestCase):
    @override
    def setUp(self) -> None:
        reporter = MultiReporter(
            GenericDiffReporter.create("diff"), CommandLineReporter()
        )
        ##approvals.set_default_reporter(reporter)

    def test(self) -> None:
        alist = ["a", "b", "c", "d", "e"]
        approvals.verify_all("letters", alist, reporter=DiffReporter())

    def test_uppercase(self) -> None:
        alist = ["a", "b", "c", "d"]
        approvals.verify_all("uppercase", alist, lambda x: f"{x} => {x.upper()}")

    def test_format_line_part1(self) -> None:
        # This is part one of a test which reproduces the issue #32
        alist = ["1", "2", "3"]
        approvals.verify_all("index", alist, reporter=DiffReporter())

    def test_format_line_part2(self) -> None:
        # This is a part two of a test that would reproduce a bug where
        # `verify_all` does not reset index when called from two different tests.
        # More details in issue #32
        alist = ["1", "2", "3"]
        approvals.verify_all("index", alist, reporter=DiffReporter())



from typing_extensions import override


class StringWrapper:
    def __init__(self) -> None:
        self.string = ""

    def append(self, text: str) -> None:
        self.string += text

    @override
    def __str__(self) -> str:
        return self.string

    @override
    def __repr__(self) -> str:
        return self.string



def copy_to_clipboard(text: str) -> None:
    """
    This acts as a wrapper to defer the importing of pyperclip util actual usage
    """
    import pyperclip  # type ignore

    pyperclip.copy(text)



import inspect
import json
import os
from copy import deepcopy
from pathlib import Path
from typing import Any, Callable, Dict, Optional, TypeVar


def get_adjacent_file(name: str) -> str:
    calling_file = inspect.stack(1)[1][1]
    directory = os.path.dirname(os.path.abspath(calling_file))
    return os.path.join(directory, name)


def write_to_temporary_file(
    text: str, name: str, file_extention_with_dot: Optional[str] = None
) -> str:
    import tempfile

    file_extention_with_dot = file_extention_with_dot or ".txt"
    with tempfile.NamedTemporaryFile(
        mode="w+b", suffix=file_extention_with_dot, prefix=name, delete=False
    ) as temp:
        temp.write(text.encode("utf-8-sig"))
        return temp.name


def to_json(object_to_verify: Any) -> str:
    return json.dumps(
        object_to_verify,
        sort_keys=True,
        indent=4,
        separators=(",", ": "),
        default=lambda o: o.__dict__,
        ensure_ascii=True,
    )


def deserialize_json_fields(a_dict: dict) -> dict:
    a_dict = deepcopy(a_dict)
    for key, val in a_dict.items():
        if isinstance(val, str) and val.startswith("{"):
            try:
                deserialized_val = json.loads(val)
            except BaseException:
                # leave field unchanged on exception
                pass
            else:
                a_dict[key] = deserialized_val
        elif isinstance(val, dict):
            a_dict[key] = deserialize_json_fields(val)
    return a_dict


def is_windows_os() -> bool:
    return os.name == "nt"


def create_empty_file(file_path: str) -> None:
    try:
        import empty_files.empty_files

        empty_files.empty_files.create_empty_file(file_path)
    except ImportError as e:
        print("Error importing empty_files", e)
        raise


def ensure_file_exists(approved_path: str) -> None:
    print("approved_path check", approved_path)
    if not os.path.isfile(approved_path):
        create_empty_file(approved_path)


def create_directory_if_needed(received_file: str) -> None:
    directory = os.path.dirname(received_file)
    if directory:
        os.makedirs(directory, exist_ok=True)


def print_grid(
    width: int, height: int, cell_print_func: Callable[[int, int], str]
) -> str:
    result = ""
    for y in range(height):
        for x in range(width):
            result += cell_print_func(x, y)
        result += "\n"
    return result


_V = TypeVar("_V")
_K = TypeVar("_K")


def filter_values(filter: Callable[[_V], bool], a_dict: Dict[_K, _V]) -> Dict[_K, _V]:
    return {k: v for k, v in a_dict.items() if filter(v)}


def append_to_file(file: Path, text: str) -> None:
    with file.open(mode="a") as f:
        f.write(text)



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from approvaltests.core.options import Options


class VerifyParameters:
    def __init__(self, options: "Options"):
        self.options = options



import threading
from abc import ABC, abstractmethod
from typing import Callable, Generic, TypeVar, cast

from typing_extensions import override

_T = TypeVar("_T")


class Wrapper(ABC, Generic[_T]):
    @abstractmethod
    def get(self) -> _T:
        pass


class SingleWrapper(Wrapper[_T]):
    def __init__(self, instance: _T):
        self.instance = instance

    @override
    def get(self) -> _T:
        return self.instance


class ThreadedWrapper(Wrapper[_T]):
    def __init__(self, generator: Callable[[], _T]):
        self.generator = generator
        self.local = threading.local()

    @override
    def get(self) -> _T:
        if not hasattr(self.local, "value"):
            self.local.value = self.generator()
        return cast(_T, self.local.value)



from typing import Any, Callable, List, Optional


def format_list(alist: List[Any], formatter: Optional[Callable], header: str) -> str:
    if formatter is None:
        formatter = FormatLineItem().print_item
    text = (header + "\n\n") if header else ""
    for i in alist:
        text += formatter(i) + "\n"
    return text


class FormatLineItem:
    def __init__(self) -> None:
        self.index = 0

    def print_item(self, item: Any) -> str:
        text = str(self.index) + ") " + str(item)
        self.index += 1
        return text



import datetime
import inspect
import traceback
from types import TracebackType
from typing import Any, Callable, ContextManager, Iterable, Optional, Type, Union

from approval_utilities.utilities.exceptions.exception_utils import to_string
from approval_utilities.utilities.stack_frame_utilities import get_class_name_for_frame
from approval_utilities.utilities.string_wrapper import StringWrapper


class Toggles:
    def __init__(self, show: bool):
        self.queries = show
        self.messages = show
        self.variables = show
        self.hour_glass = show
        self.markers = show
        self.events = show


def _is_iterable(arg: Any) -> bool:
    return isinstance(arg, Iterable) and not isinstance(arg, str)


def print_type(value: Any) -> str:
    return f"<{type(value).__name__}>"


class LoggingInstance:
    def __init__(self) -> None:
        self.log_stack_traces = True
        self.toggles = Toggles(True)
        self.previous_timestamp: Optional[datetime.datetime] = None
        self.logger: Callable[[str], None] = lambda t: print(t, end="")
        self.tabbing: int = 0
        self.counter = 0
        self.log_with_timestamps = True
        self.timer: Callable[[], datetime.datetime] = datetime.datetime.now

    def log_to_string(self) -> StringWrapper:
        buffer = StringWrapper()
        self.logger = buffer.append
        self.log_with_timestamps = False
        self.log_stack_traces = False
        return buffer

    def indent(self) -> ContextManager[None]:
        class Indent:
            def __init__(self, log: "LoggingInstance") -> None:
                self.log = log

            def __enter__(self) -> None:
                self.log.tabbing += 1

            def __exit__(
                self,
                exc_type: Optional[Type[BaseException]],
                exc_val: Optional[BaseException],
                exc_tb: Optional[TracebackType],
            ) -> None:
                self.log.tabbing -= 1

        return Indent(self)

    def use_markers(
        self,
        parameter_text: Optional[Union[str, Callable[[], str]]] = None,
        additional_stack: int = 0,
    ) -> ContextManager[None]:
        class Nothing:
            def __enter__(self) -> None:
                pass

            def __exit__(
                self,
                exc_type: Optional[type],
                exc_val: Optional[BaseException],
                exc_tb: Optional[TracebackType],
            ) -> None:
                pass

        if not self.toggles.markers:
            return Nothing()

        class Markers:
            def __init__(
                self, log: "LoggingInstance", method_name: str, filename: str
            ) -> None:
                self.log = log
                self.method_name = method_name
                self.filename = filename

            def __enter__(self) -> None:
                expected = f"-> in: {self.method_name}({self.get_parameters(False)}) in {self.filename}"
                self.log.log_line(expected)
                self.log.tabbing = self.log.tabbing + 1

            def __exit__(
                self,
                exc_type: Optional[type],
                exc_val: Optional[BaseException],
                exc_tb: Optional[TracebackType],
            ) -> None:
                self.log.tabbing = self.log.tabbing - 1
                expected = f"<- out: {self.method_name}({self.get_parameters(True)})"
                self.log.log_line(expected)

            def get_parameters(self, is_exit: bool) -> str:
                if callable(parameter_text):
                    return parameter_text()
                elif parameter_text is None or is_exit:
                    return ""
                else:
                    return str(parameter_text)

        stack_position = 1 + additional_stack
        stack = inspect.stack(stack_position)[2]
        method_name = stack.function

        filename = get_class_name_for_frame(stack)
        return Markers(self, method_name, filename)

    def log_line(self, text: str, use_timestamps: bool = True) -> None:
        if self.counter != 0:
            self.logger("\n")
            self.counter = 0
        timestamp = self.get_timestamp() if use_timestamps else ""
        output_message = f"{timestamp}{self.get_tabs()}{text}\n"
        self.logger(output_message)

    def get_timestamp(self) -> str:
        timestamp = ""
        if self.log_with_timestamps:
            time1: datetime.datetime = self.timer()
            time = time1.strftime("%Y-%m-%dT%H:%M:%SZ")
            diff_millseconds = 0
            if self.previous_timestamp != None:
                delta = time1 - self.previous_timestamp
                diff_millseconds = int((delta).total_seconds() * 1000)
            diff = diff_millseconds
            diff_display = f" ~{diff:06}ms"
            timestamp = f"[{time} {diff_display}] "
            self.previous_timestamp = time1
        return timestamp

    def hour_glass(self) -> None:
        if not self.toggles.hour_glass:
            return

        self.increment_hour_glass_counter()
        if self.counter == 1:
            self.logger(f"{self.get_tabs()}.")
        elif self.counter == 100:
            self.logger("10\n")
            self.counter = 0
        elif self.counter % 10 == 0:
            digit = int(self.counter / 10)
            self.logger(f"{digit}")
        else:
            self.logger(".")

    def get_tabs(self) -> str:
        return "  " * self.tabbing

    def increment_hour_glass_counter(self) -> None:
        self.counter = self.counter + 1

    def variable(self, name: str, value: Any, show_types: bool = False) -> None:
        if not self.toggles.variables:
            return

        def to_type(value: Any, spacing: str = " ") -> str:
            return f"{spacing}{print_type(value)}" if show_types else ""

        if _is_iterable(value):
            self.log_line(
                f"variable: {name}{to_type(value, spacing='')}.length = {len(value)}"
            )
            with self.indent():
                for i, v in enumerate(value):
                    self.logger(f"{self.get_tabs()}{name}[{i}] = {v}{to_type(v)}\n")

        else:
            self.log_line(f"variable: {name} = {value}{to_type(value)}")

    def event(self, event_name: str) -> None:
        if not self.toggles.events:
            return
        self.log_line(f"event: {event_name}")

    def query(self, query_text: str) -> None:
        if not self.toggles.queries:
            return
        self.log_line(f"Sql: {query_text}")

    def message(self, message: str) -> None:
        if not self.toggles.messages:
            return
        self.log_line(f"message: {message}")

    def warning(
        self,
        text: Union[str, BaseException] = "",
        exception: Optional[BaseException] = None,
    ) -> None:
        if isinstance(text, Exception):
            temp = ""
            if exception:
                temp = str(exception)
            exception = text
            text = temp

        warning_stars = "*" * 91
        self.log_line(warning_stars, use_timestamps=False)
        if self.log_with_timestamps:
            self.log_line("", use_timestamps=True)
        if text:
            self.log_line(f"Message:{text}", use_timestamps=False)
        if exception:
            if self.log_stack_traces:
                format_exception = traceback.format_exception(
                    None, exception, exception.__traceback__
                )
                stack_trace = "".join(format_exception)
            else:
                stack_trace = to_string(exception)
            self.log_line(stack_trace, use_timestamps=False)
        self.log_line(warning_stars, use_timestamps=False)

    def show_queries(self, show: bool) -> None:
        self.toggles.queries = show

    def show_all(self, show: bool) -> None:
        self.toggles = Toggles(show)

    def show_messages(self, show: bool) -> None:
        self.toggles.messages = show

    def show_variables(self, show: bool) -> None:
        self.toggles.variables = show

    def show_hour_glass(self, show: bool) -> None:
        self.toggles.hour_glass = show

    def show_markers(self, show: bool) -> None:
        self.toggles.markers = show

    def show_events(self, show: bool) -> None:
        self.toggles.events = show
